@@ -144,6 +144,7 @@ class GPS(object):
         position_dict = info_dict["gps"]["position"]
 
         self.position = Vector3.from_dict(position_dict)
+        self.velocity = Vector3.from_dict(info_dict["gps"]["velocity"])
         self.__info_object_reference = info_object_reference
 
 
@@ -288,7 +289,7 @@ class RobotInfo(object):
         self.radar = Radar(self, info_dict)
 
         self.timestamp = info_dict["timestamp"]
-        self.isIt = info_dict["isIt"]
+        self.is_it = info_dict["isIt"]
         self.gamemode = info_dict["gameMode"]
 
 
@@ -303,7 +304,7 @@ class RobotConnection:
     """
 
     __SEND_INTERVAL = 0.01  # in seconds
-    __GET_INTERVAL = 0.01
+    __GET_INTERVAL = 0.005
     __QUEUE_INTERVAL = 0.001
 
     __INFO_PIPE_NAME = "RobotInfoPipe"
@@ -320,10 +321,20 @@ class RobotConnection:
             self.__platform = "POSIX"
 
         self.__event_buffer = []  # where events that need to be sent are stored
+        self.__event_buffer_lock = threading.Lock()  # lock for threads wanting to access event_buffer
+
         self.info = None # type: RobotInfo
         self.__info_dict = None
+
+        self.__info_coherent_lock = False  # allows user to prevent info from being updated
+        self.__hidden_dict = None # where state is stored while info is locked
+        self.__hidden_info = None
+
         self.__should_destroy = False  # should this connection end
-        self.__event_buffer_lock = threading.Lock()  # lock for threads wanting to access event_buffer
+
+        self.__connection_lock = threading.Condition()
+        self.__send_connected = False
+        self.__get_connected = False
 
     def __queue_event(self, event_text):
 
@@ -421,6 +432,11 @@ class RobotConnection:
 
         win32pipe.ConnectNamedPipe(event_queue_pipe, None)
 
+        self.__connection_lock.acquire()
+        self.__send_connected = True
+        self.__connection_lock.notify_all()
+        self.__connection_lock.release()
+
         while len(self.__event_buffer) > 0 or not self.__should_destroy:
 
             with self.__event_buffer_lock:
@@ -428,13 +444,21 @@ class RobotConnection:
                 message = self.__EVENT_SEPARATOR.join(self.__event_buffer) + "\n"
                 self.__event_buffer = []
 
-            win32file.WriteFile(event_queue_pipe, bytes(message, "ascii"))
+            if message != "\n":
+
+                win32file.WriteFile(event_queue_pipe, bytes(message, "ascii"))
 
             time.sleep(self.__SEND_INTERVAL)
 
     def __send_buffer_thread_posix(self):
 
         fifo = utility.PosixFifo(self.__EVENT_QUEUE_PIPE_NAME, "w")
+
+        self.__connection_lock.acquire()
+        self.__send_connected = True
+        self.__connection_lock.notify_all()
+        self.__connection_lock.release()
+
         while len(self.__event_buffer) > 0 or not self.__should_destroy:
 
             with self.__event_buffer_lock:
@@ -442,7 +466,9 @@ class RobotConnection:
                 message = self.__EVENT_SEPARATOR.join(self.__event_buffer) + "\n"
                 self.__event_buffer = []
 
-            fifo.stream.write(message)
+            if message != "\n":
+
+                fifo.stream.write(message)
 
             fifo.temp_close()
             time.sleep(self.__SEND_INTERVAL)
@@ -477,8 +503,29 @@ class RobotConnection:
 
             self.info = tmp_info
 
+            if not self.__get_connected:
+
+                self.__connection_lock.acquire()
+                self.__get_connected = True
+                self.__connection_lock.notify_all()
+                self.__connection_lock.release()
+
             time.sleep(self.__GET_INTERVAL)
 
+    def lock_info(self):
+
+        self.__info_coherent_lock = True
+
+        self.__hidden_info = self.info
+        self.__hidden_dict = self.__info_dict
+
+    def unlock_info(self):
+
+        self.info = self.__hidden_info
+        self.__info_dict = self.__hidden_dict
+
+        self.__info_coherent_lock = False
+
     def __get_robot_state_thread_posix(self):
 
         fifo = utility.PosixFifo(self.__INFO_PIPE_NAME, "r")
@@ -487,16 +534,32 @@ class RobotConnection:
 
             info_text = utility.get_most_recent_message_posix(fifo)
 
-            tmp_dict = json.loads(info_text)
-            self.__info_dict = tmp_dict
+            if info_text != "":
 
-            tmp_info = RobotInfo(self.__info_dict)
+                tmp_dict = json.loads(info_text)
 
-            if self.info:
+                tmp_info = RobotInfo(tmp_dict)
 
-                tmp_info.coordinates_are_inverted = self.info.coordinates_are_inverted
+                if self.info:
 
-            self.info = tmp_info
+                    tmp_info.coordinates_are_inverted = self.info.coordinates_are_inverted
+
+                if self.__info_coherent_lock:
+
+                    self.__hidden_info = tmp_info
+                    self.__hidden_dict = tmp_dict
+
+                else:
+
+                    self.info = tmp_info
+                    self.__info_dict = tmp_dict
+
+            if not self.__get_connected:
+
+                self.__connection_lock.acquire()
+                self.__get_connected = True
+                self.__connection_lock.notify_all()
+                self.__connection_lock.release()
 
             time.sleep(self.__GET_INTERVAL)
 
@@ -509,9 +572,11 @@ class RobotConnection:
         None
         """
 
+        self.__connection_lock.acquire()
+
         if self.__platform == "Windows":
 
-            send_thread = threading.Thread(target=self.__send_buffer_thread_windows)
+            send_thread = threading.Thread(target=self.__send_buffer_thread_windows,)
             get_thread = threading.Thread(target=self.__get_robot_state_thread_windows)
 
         else:
@@ -521,4 +586,8 @@ class RobotConnection:
 
         send_thread.start()
         get_thread.start()
-        time.sleep(1)
+
+        while not self.__send_connected or not self.__get_connected:
+            self.__connection_lock.wait()
+
+        self.__connection_lock.release()
